# Third party package
import click

# pass_gen packages
import pass_gen

def version(ctx, param, value):
    '''
    Show version
    '''
    if not value or ctx.resilient_parsing:
        return
    click.echo(pass_gen.__version__)
    ctx.exit()

@click.command()
@click.option('-v', '--version', is_flag=True, callback=version,
                expose_value=False, is_eager=True, help='Show version')
@click.option('-l', '--length', type=int, default=12, metavar='', help='Password length [8 - 16]')
def cli(length):
    if length in range(8, 17):
        password = pass_gen.generate(length)
        click.echo(password)
    else:
        click.echo('Password length must be between 8-16.')

if __name__ == '__main__':
    cli(12)


# Python lib
from random import choice

__version__ = '0.0.5'

NUMBERS = ''.join([chr(i) for i in range(48, 58)])
UPPERCASE_LETTERS = ''.join([chr(i) for i in range(65, 91)])
LOWERCASE_LETTERS = ''.join([chr(i) for i in range(97, 123)])
MISC_LETTERS = '!#$%&^*?'
PASS_LIST = [NUMBERS, UPPERCASE_LETTERS, LOWERCASE_LETTERS, MISC_LETTERS]

def generate(length = 12) -> str:
    '''Generates random password'''
    valid = False

    while not valid:
        password = ''
        while len(password) <= length:
            choiced_letters = choice(PASS_LIST)
            password += choice(choiced_letters)
        valid = _validate_password(password)
    
    return password

def _validate_password(password) -> bool:
    '''
    Validate password have at least one number, uppercase letter,
    lowercase letter, misc letter
    '''
    number = False
    uppercase_letter = False
    lowercase_letter = False
    misc_letter = False

    for letter in password:
        if letter in NUMBERS:
            number = True
            continue
        if letter in UPPERCASE_LETTERS:
            uppercase_letter = True
            continue
        if letter in LOWERCASE_LETTERS:
            lowercase_letter = True
            continue
        if letter in MISC_LETTERS:
            misc_letter = True
            continue
    
    if number and uppercase_letter and lowercase_letter and misc_letter:
        return True
    return False